#include "ShaderConstants.fxh"
#include "util.fxh"

struct PS_Input
{
	float4 position : SV_Position;

#ifndef BYPASS_PIXEL_SHADER
	lpfloat4 color : COLOR;
	snorm float2 uv0 : TEXCOORD_0_FB_MSAA;
	snorm float2 uv1 : TEXCOORD_1_FB_MSAA;
#endif

#ifdef FOG
	float4 fogColor : FOG_COLOR;
#endif
};

struct PS_Output
{
	float4 color : SV_Target;
};

ROOT_SIGNATURE


//トーンマップ
#define ESBE_TONEMAP
#define saturation 1.333//色の強さ
#define exposure 1.075//雨天時と晴天時の演出の差
#define brightness 0.750//全体的な明るさ
#define gamma 1.900//描写の「硬さ」
#define contrast 1.97//彩度
//=*=-*-=*=


float filmic_curve(float x) {

// Shoulder strength
float A = 0.22;
// Linear strength
float B = 0.5;
// Linear angle
float C = 0.15 * brightness;
// Toe strength
float D = 0.4 * gamma;
// Toe numerator
float E = 0.01 * contrast;
// Toe denominator
float F = 0.2;

return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
}

float3 ESBEmapping(float3 clr) {
float W = 1.0 / exposure;

#ifdef ESBE_TONEMAP

float Luma = dot(clr, float3(0.0, 0.3, 0.3));
float3 Chroma = clr - Luma;
clr = (Chroma * saturation) + Luma;
  clr = float3(filmic_curve(clr.r), filmic_curve(clr.g), filmic_curve(clr.b)) / filmic_curve(W);
#endif
return clr;
}


void main(in PS_Input PSInput, out PS_Output PSOutput)
{
#ifdef BYPASS_PIXEL_SHADER
    PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
    return;
#else

float torchlightaffects = (1.0 - PSInput.uv1.x);


//光源
float3 ESBE_LIGHT = float3(0.700,0.500,0.300);
//=*=-*-=*=


#if USE_TEXEL_AA
	float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0 );
#else
	float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0f;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

#if defined(BLEND)
	diffuse.a *= PSInput.color.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse = diffuse * TEXTURE_1.Sample(TextureSampler1, PSInput.uv1);
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = PSInput.color.a;
	#endif

	diffuse.rgb *= PSInput.color.rgb;
#else
	float2 uv = PSInput.color.xy;
	diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
	diffuse.rgb *= PSInput.color.aaa;
	diffuse.a = 1.0f;
#endif

#ifdef FOG
	float weather = smoothstep(0.8,1.0,FOG_CONTROL.y);
#else
	float weather = 1.0;
#endif


//影の境界(数が大きいほどぼやける)
float blur = 0.005;
//=*=-*-=*=


float s_amount = lerp(0.0,0.45,smoothstep(0.87 + blur,0.87 - blur,PSInput.uv1.y))*weather;
float nolight = (1.0 - PSInput.uv1.x);


//影の色
float3 ESBEshadow = float3(0.0,0.0,0.0);
//=*=-*-=*=


diffuse.rgb = lerp(diffuse.rgb, ESBEshadow.rgb, s_amount * nolight * torchlightaffects);

#ifdef FOG
	diffuse.rgb = lerp( diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a );
#endif

//夕焼け
float DUSK = 0.3;//夕焼けの色の強さ
float dusk1 = 0.25;//境界位置
float dusk2 = 0.25;//境界ブラー

float daylight = TEXTURE_1.Sample(TextureSampler1, float2(0.0, 1.0)).r;
float dusk3 = DUSK*smoothstep(dusk1,dusk1+dusk2, PSInput.uv1.y)*weather;
float dusk = smoothstep(0.1,0.5,abs(smoothstep(0.4,0.8,daylight)-0.5))*dusk3+(1.0-dusk3);
diffuse.rgb *= float3(2.0-dusk,1.0,dusk);
//=*=-*-=*=

diffuse.rgb = ESBEmapping(diffuse.rgb);
diffuse.rgb += ESBE_LIGHT.rgb*max(0.0,PSInput.uv1.x-0.5f);

PSOutput.color = diffuse;

#ifdef VR_MODE
	// On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to
	// the lowest 8 bit value.
	PSOutput.color = max(PSOutput.color, 1 / 255.0f);
#endif

#endif // BYPASS_PIXEL_SHADER
}
